import unittest
from src.value_tree import ValueNode

class CreateNode(unittest.TestCase):
    def test_single_node_value(self):
        tree = ValueNode("root", 10)
        self.assertEqual(tree.key,"root")
        self.assertEqual(tree.value,10)

    def test_add_new_leaf_value(self):
        tree = ValueNode("root")
        tree.try_new_leaf("node 1", (2,"foo"))
        self.assertEqual(tree["node 1"].value, (2,"foo"))

if __name__ == "__main__":
    unittest.main()


import unittest
from src.value_tree import ValueNode

class TraverseTree(unittest.TestCase):
    def setUp(self) -> None:
        self.tree = ValueNode("root")
        self.tree.new_leaf("fruits")
        self.tree["fruits"].new_leaf("red")
        self.tree["fruits"].new_leaf("yellow")
        self.tree["fruits"].new_leaf("purple")
        self.tree["fruits"].new_leaf("green")
        
        self.tree["fruits"]["red"].new_leaf("apple")
        self.tree["fruits"]["red"].new_leaf("strawberry")

        self.tree["fruits"]["yellow"].new_leaf("banana")
        self.tree["fruits"]["yellow"].new_leaf("pineapple")
        
        self.tree["fruits"]["purple"].new_leaf("berry")

        self.tree["fruits"]["green"].new_leaf("pear")
        self.tree["fruits"]["green"].new_leaf("apple")

        self.tree.new_leaf("tech companies")
        self.tree["tech companies"].new_leaf("apple", 345)
        self.tree["tech companies"].new_leaf("microsoft", 243)
        self.tree["tech companies"].new_leaf("google",348)
        self.tree["tech companies"].new_leaf("meta",145)


    def test_traverse_node_shortcut(self):
        self.tree.try_new_leaf("shortcut")
        self.assertEqual(self.tree["shortcut"], self.tree._children["shortcut"])

    def test_traverse_multiple_nodes(self):
        google = self.tree["tech companies"]["google"]
        self.assertEqual(google.key, "google")
        self.assertEqual(google.value,348)

        banana = self.tree["fruits"]["yellow"]["banana"]
        self.assertEqual(banana.key, "banana")
        self.assertEqual(banana.value, None)

    def test_traverse_to_parent_with_arrow(self):
        green = self.tree["fruits"]["green"]["apple"]["__<-__"]
        self.assertEqual(green.key,"green")

    def test_one_level_wildcard(self):
        berry = self.tree["fruits"]["__*__"]["berry"].tolist()
        self.assertEqual(len(berry), 1)
        self.assertEqual(berry[0].key, "berry")
        self.assertEqual(berry[0]["__<-__"].key, "purple")
    
    def test_wildcard_first_or_nonode(self):
        berry = self.tree["fruits"]["__*__"]["berry"]["__?__"]
        mango = self.tree["fruits"]["__*__"]["mango"]["__?__"]

        self.assertEqual(berry.key, "berry")
        self.assertEqual(mango,None)

if __name__ == "__main__":
    unittest.main()



import unittest
from src import Node

class CreateNodes(unittest.TestCase):
    def test_single_node(self):
        tree = Node("root")
        self.assertEqual(tree.key,"root") 

    def test_add_new_leaf(self):
        tree = Node("root")
        tree.new_leaf("node 1")
        self.assertEqual(tree["node 1"].parent, tree,
            "The parent of the new leaf should be the creator node")
        self.assertEqual(tree["node 1"].key, "node 1")

    def test_remove_node(self):
        tree = Node("root")
        tree.new_leaf("node 1")
        tree.new_leaf("node 2")
        self.assertEqual(len(tree._children),2)

        del tree["node 1"]
        self.assertEqual(len(tree._children),1)

        del tree["node 2"]
        self.assertEqual(tree._children, {})    

    def test_try_new_leaf(self):
        tree = Node("root")
        first = tree.try_new_leaf("node 1")
        second = tree.try_new_leaf("node 1")

        self.assertEqual(first, True)
        self.assertEqual(second, False)

        self.assertNotEqual(tree["node 1"],None)


import unittest
from src import ValueNode
from src.specialClasses.base_node import NoNode

class NoNodes(unittest.TestCase):
    def test_get_unexistent_node_returns_nonode(self):
        tree = ValueNode("root")
        is_nonode = isinstance(tree["ghost node"],NoNode)
        self.assertEqual(is_nonode, True)
    
    def test_nonode_is_None(self):
        tree = ValueNode("root")
        self.assertEqual(tree["unexistent node"], None)
        self.assertEqual(tree["unexistent node"].key, None)
        self.assertEqual(tree["unexistent node"].value, None)

    def test_chained_nonodes(self):
        tree = ValueNode("root")
        nonode = tree["unexisting node"]["ghost node"]["magic node"]
        is_nonode = isinstance(nonode,NoNode)
        self.assertEqual(is_nonode, True)

    def test_add_nonode(self):
        tree = ValueNode("root")
        tree["ethereal node"].create_all()
        self.assertEqual(tree["ethereal node"].key, "ethereal node")

    def test_add_multiple_nonodes(self):
        tree = ValueNode("root")
        tree["new 1"]["new 2"].create_all()

        self.assertNotEqual(tree["new 1"]["new 2"], None)
        self.assertEqual(tree["new 1"]["new 2"].parent.key, "new 1")


if __name__ == "__main__":
    unittest.main()



from __future__ import annotations
from collections import deque
from typing import TYPE_CHECKING
if TYPE_CHECKING:
    from base_node import Node


class TreeIterator:
    '''Alow to iterate a tree with both deep first and breadth first search algorithms'''
    def __init__(self,root_node: Node, deep_first=True):
        self.queue = deque()
        self.queue.append((root_node,0))
        self.deep_first = deep_first

    def _pop_next_node(self):
        if self.deep_first:
            return self.queue.pop()
        else:
            return self.queue.popleft()
    
    def __iter__(self):
        return self

    def __next__(self):
        if not self.queue:
            raise StopIteration

        (node, depth) = self._pop_next_node()
        if node.children:
            for _,child in node.children.items():
                self.queue.append((child, depth + 1))
        return (node,depth)
